#include "project.h"
#include "stdio.h"
#include "stdlib.h"
#include "tty_utils.h"
#include "i2c_utils.h"

static uint8_t  i2c_slave_address = 0x08 ; 
static int      i2c_delay    = 30 ; /* default 30ms */

void i2c_set_slave_address(uint8_t slave_address)
{
    i2c_slave_address = slave_address ;
}

int i2c_test_address(uint8_t address) 
{
    int result = 0 ;
    uint8_t errStatus = I2C_MSTR_NO_ERROR ;
    
    errStatus = I2C_MasterSendStart(address, 1) ; /* read access */   
    if (errStatus == I2C_MSTR_NO_ERROR) {
        result = 1 ;
    }
    errStatus = I2C_MasterSendStop() ;    
    return(result) ;
}

int i2c_read_byte(uint8_t *value) 
{
    uint8 errStatus = I2C_MSTR_NO_ERROR ;
    errStatus = I2C_MasterSendStart(i2c_slave_address, 1) ; /* read access */
    if (errStatus == I2C_MSTR_NO_ERROR) {
        CyDelay(i2c_delay) ;
        *value = I2C_MasterReadByte(I2C_ACK_DATA) ;
    } else {
        print("I2C Send Start Failed!\n\r") ;
    }
    if (errStatus != I2C_MSTR_NO_ERROR) {
        print("I2C Read Byte Failed!\n\r") ;
    }
    errStatus = I2C_MasterSendStop() ;
    return(errStatus) ;
}

int i2c_write_byte(uint8_t value) 
{
    uint8 errStatus = I2C_MSTR_NO_ERROR ;
    errStatus = I2C_MasterSendStart(i2c_slave_address, 0) ; /* write access */
    if (errStatus == I2C_MSTR_NO_ERROR) {
        CyDelay(i2c_delay) ;
        errStatus = I2C_MasterWriteByte(value) ;
    } else {
        print("I2C Send Start Failed!\n\r") ;
    }
    if (errStatus != I2C_MSTR_NO_ERROR) {
        print("I2C Write Byte Failed!\n\r") ;
    }
    errStatus = I2C_MasterSendStop() ;
    return(errStatus) ;    
}

int i2c_read_reg(uint8_t reg_address, uint8_t *value) 
{
    uint8 errStatus = I2C_MSTR_NO_ERROR ;
    errStatus = I2C_MasterSendStart(i2c_slave_address, 0) ; /* write access */
    if (errStatus == I2C_MSTR_NO_ERROR) {
        CyDelay(i2c_delay) ;
        errStatus = I2C_MasterWriteByte(reg_address) ;
    } else {
        print("I2C Send Start Failed!\n\r") ;
    }
    if (errStatus == I2C_MSTR_NO_ERROR) {
        CyDelay(i2c_delay) ;
        errStatus = I2C_MasterSendRestart(i2c_slave_address, 1) ; /* read access */
    } else {
        print("I2C Write Reg Address Failed\n\r") ;
    }
    if (errStatus == I2C_MSTR_NO_ERROR) {
        CyDelay(i2c_delay) ;
        *value = I2C_MasterReadByte(I2C_NAK_DATA) ;
    }
    if (errStatus != I2C_MSTR_NO_ERROR) {
        print("I2C Read Reg Data Failed!\n\r") ;
    }
    errStatus = I2C_MasterSendStop() ;  
    return( errStatus ) ;
}

int i2c_write_reg(uint8_t reg_address, uint8_t value) 
{
    uint8 errStatus = I2C_MSTR_NO_ERROR ;
    errStatus = I2C_MasterSendStart(i2c_slave_address, 0) ; /* write access */
    if (errStatus == I2C_MSTR_NO_ERROR) {
        CyDelay(i2c_delay) ;
        errStatus = I2C_MasterWriteByte(reg_address) ;
    } else {
        print("I2C Send Start Failed!\n\r") ;
    }
    if (errStatus == I2C_MSTR_NO_ERROR) {
        CyDelay(i2c_delay) ;
        errStatus = I2C_MasterWriteByte(value) ;
    }
    if (errStatus != I2C_MSTR_NO_ERROR) {
        print("I2C Write Reg Data Failed!\n\r") ;
    }
    errStatus = I2C_MasterSendStop() ;  
    return( errStatus ) ;
}

int i2c_read_regs(uint8_t reg_address, uint8_t *value, int num) 
{
    int i ;
    uint8 errStatus = I2C_MSTR_NO_ERROR ;

    errStatus = I2C_MasterSendStart(i2c_slave_address, 0) ; /* write access */
    if (errStatus == I2C_MSTR_NO_ERROR) {
        CyDelay(i2c_delay) ;
        errStatus = I2C_MasterWriteByte(reg_address) ;
    } else {
        print("I2C Send Start Failed!\n\r") ;
    }
    if (errStatus == I2C_MSTR_NO_ERROR) {
        CyDelay(i2c_delay) ;
        errStatus = I2C_MasterSendRestart(i2c_slave_address, 1) ; /* read access */
    } else {
        print("I2C Write Reg Address Failed\n\r") ;
    }
    if (errStatus == I2C_MSTR_NO_ERROR) {
        for (i = 0 ; i < num - 1 ; i++ ){
            CyDelay(i2c_delay) ;
            value[i] = I2C_MasterReadByte(I2C_ACK_DATA) ;
        }
        if (errStatus == I2C_MSTR_NO_ERROR) {
            value[i] = I2C_MasterReadByte(I2C_NAK_DATA) ;
        } else {
            print("I2C Read Reg Data Failed!\n\r") ; 
        }
    } else {
        print("I2C Send Restart Failed!\n\r") ;
    }
    errStatus = I2C_MasterSendStop() ;  

    return( errStatus ) ;    
}

int i2c_read_regs_w_buf(uint8_t regaddress, uint8_t *value, int num) 
{
    uint8 errStatus = I2C_MSTR_NO_ERROR ;

    errStatus = I2C_MasterSendStart(i2c_slave_address, 0) ; /* write access */
    if (errStatus == I2C_MSTR_NO_ERROR) {
        CyDelay(i2c_delay) ;
        errStatus = I2C_MasterWriteByte(regaddress) ;
    } else {
        print("I2C Send Start Failed!\n\r") ;
    }
    if (errStatus == I2C_MSTR_NO_ERROR) {
        CyDelay(i2c_delay) ;
        errStatus = I2C_MasterReadBuf(i2c_slave_address, value, num, 
            I2C_MODE_REPEAT_START | I2C_MODE_COMPLETE_XFER ) ;
    }
    return( errStatus) ;
}

int i2c_write_regs_w_buf(uint8_t regaddress, uint8_t *value, int num) 
{
    uint8 errStatus = I2C_MSTR_NO_ERROR ;
    uint8_t *tmp ;
    int i ;
    
    tmp = (uint8_t *)malloc(sizeof(uint8_t) * (num + 1)) ;
    if (tmp == 0) {
        print("Not Enough Memory!") ;
        return I2C_MSTR_ERR_ABORT_START_GEN ;
    }
    tmp[0] = regaddress ;
    for (i = 0 ; i < num ; i++) {
        tmp[i+1] = value[i] ;
    }
    
    errStatus = I2C_MasterWriteBuf(i2c_slave_address, tmp, num+1,
        I2C_MODE_COMPLETE_XFER) ;
    
    free(tmp) ;
    
    return( errStatus ) ;
}